/*
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *     
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "global.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

GtkWidget *dlgFicheros=NULL;
GtkWidget *dlgespera=NULL;
GtkWidget *dlgEnvia=NULL;
DISPOS *inicio=NULL;
char *fichero=NULL;
int estado;

void
on_esperar_conexiones_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	dlgespera=create_dlgEspera();

	gtk_widget_show_now(dlgespera);
	/* Le dejo hacer sus eventos antes de esperar conexiones || realize events*/
	while (gtk_events_pending ())
		gtk_main_iteration ();
	
	cancel=0;
	espera_conexiones();

	if (dlgespera != NULL)    /* Ya ha sido cerrado por otro evento || It was closed for another event */
		gtk_widget_destroy(dlgespera);
}

void
on_cmdExaminar_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	dlgFicheros = create_dlgFicheros ();
	gtk_widget_show (dlgFicheros);
}

void
on_dlgCancelar_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(dlgFicheros);
}


void
on_dlgAceptar_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *txtfichero=txtFichero;
        
	fichero=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dlgFicheros));		/*Pillo la ruta y el fichero || get the file and rute*/
	gtk_entry_set_text(GTK_ENTRY(txtfichero),fichero);
	gtk_widget_destroy(dlgFicheros);
}

void
on_cmdDetecta_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkComboBox *cbElijeDisp;
	inquiry_info *ii = NULL; //Almacena la lista de dispositivos detectados durante el inquiry || a device list captured in inquiry
	int max_rsp, num_rsp; //Numero de respuestas/dispositivos detectados || number of devices
	int dev_id; //Identificador del adaptador Bluetooth local || local bluetooth id
	int socket; //Socket HCI;
	int len, i;
	char MAC_dev[20]; //Direccion MAC del dispositivo detectado	|| MAC of remote device
	char nombre_dev[248]; //Nombre del dispositivo detectado	|| name of remote device
	DISPOS *nuevo=NULL;
	DISPOS *ultimo=NULL;
	GtkWidget *dlgBuscando;
	GtkWidget *dlgError;
	int error=FALSE;
	
	dlgBuscando=create_dlgBuscando();
	gtk_widget_show_now(dlgBuscando);
	/* Le dejo hacer sus eventos antes de buscar dispositivos || do events */
	while (gtk_events_pending ())
		gtk_main_iteration ();

	cbElijeDisp=GTK_COMBO_BOX(button);		
	
	//Obtenemos el identificador del adaptador local Bluetooth	|| get id of local bluetooth device
	dev_id = busca_disp();
	
	if (dev_id < 0)
	{
		dlgError = gtk_message_dialog_new (NULL,
					  GTK_DIALOG_DESTROY_WITH_PARENT,
					  GTK_MESSAGE_ERROR,
					  GTK_BUTTONS_CLOSE,
					  "Error: El dispositivo local no esta disponible.");
		gtk_dialog_run (GTK_DIALOG (dlgError));
		gtk_widget_destroy (dlgError);
		g_print("[!] Error. Dispositivo Bluetooth local no disponible.");
		
		error=TRUE;
	}
	
	//Abrimos un socket local HCI || open HCI local socket 
	socket = hci_open_dev(dev_id);
	if (socket < 0 && error == FALSE)
	{
		dlgError = gtk_message_dialog_new (NULL,
					  GTK_DIALOG_DESTROY_WITH_PARENT,
					  GTK_MESSAGE_ERROR,
					  GTK_BUTTONS_CLOSE,
					  "Error: Fallo al intentar abrir socket HCI");
		gtk_dialog_run (GTK_DIALOG (dlgError));
		gtk_widget_destroy (dlgError);
		g_print("[!] Error. Fallo al intentar abrir socket HCI.\n Failed to open the socket.\n");
			
		error=TRUE;
	}
		
	if (error==FALSE)
	{
		//Inicializamos algunas variables	|| initialize some variables
		len = 10;	//El tiempo de inquiry por dispositivo || max time to inquiry a remote device
		max_rsp = 255;	//Se pueden detectar a lo sumo 255 dispositivos || max number of remote device to detect
	
		//Creamos la lista de dispositivos detectados con hci_inquiry || make a list of devices
		ii = (inquiry_info*)malloc(max_rsp * sizeof(inquiry_info));
		
		g_print("Detectando dispositivos...\n\n");
		
		cancel=0;
		
		//La bandera IREQ_CACHE_FLUSH permite que la caché sea limpiada antes de buscar nuevos dispositivos.
		//The flag IREQ_CACHE_FLUSH let to clean cache before search new devices
		num_rsp = hci_inquiry(dev_id, len, max_rsp, NULL, &ii, IREQ_CACHE_FLUSH);
	}
	
	/* Le dejo hacer sus eventos antes de buscar dispositivos || do events*/
	while (gtk_events_pending ())
		gtk_main_iteration ();
	
	if (cancel==1)	/*Se cancelo la accion || the action was cancel */
	{
		reset_dispos(cbElijeDisp);
		free(ii);
		close(socket);
		gtk_combo_box_set_active(cbElijeDisp,0);
		gtk_widget_destroy(dlgBuscando);     
				
		error=TRUE;
	}
	
	if(num_rsp < 0 && error==FALSE)
	{
		dlgError = gtk_message_dialog_new (NULL,
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_ERROR,
						 GTK_BUTTONS_CLOSE,
						"Error: Fallo al intentar buscar dispositivos cercanos");
		gtk_dialog_run (GTK_DIALOG (dlgError));
		gtk_widget_destroy (dlgError);
		
		g_print("[!] Error. Fallo al intentar hci_inquiry.\n Failed on hci_inquiry\n.");
		
		error=TRUE;
	}

	if (error==FALSE)
	{
		/* Si la lista ya existe la elimino || if exist some list, then remove the list*/
		reset_dispos (cbElijeDisp);
	
		//Para cada una de las respuestas obtenidas durante el inquiry obtenemos el nombre del dispositivo
		//For each responses on inquiry we get the name of device
		for(i=0; i<num_rsp; i++)
		{
			ba2str(&(ii+i)->bdaddr, MAC_dev);
			memset(nombre_dev, 0, sizeof(nombre_dev));
			if(hci_read_remote_name(socket, &(ii+i)->bdaddr, sizeof(nombre_dev), nombre_dev, 0) < 0)
				strcpy(nombre_dev, "[Desconocido]");
	
			g_print("Dispositivo (%d) encontrado:\n\tMAC: %s\n\tNombre: %s\n\n", i+1, MAC_dev, nombre_dev);
			gtk_combo_box_insert_text (cbElijeDisp, i, nombre_dev);
			
			/*Añado los datos a la lista enlazada || add data to the list*/
			nuevo=(DISPOS *)malloc(sizeof(DISPOS));
			nuevo->next=NULL;
			strcpy(nuevo->MAC_dev,MAC_dev);
			strcpy(nuevo->nombre_dev,nombre_dev);
			if (inicio==NULL)	/*Si es el primer elemento || if the first element*/
			{
				inicio=nuevo;
				ultimo=nuevo;
			}
			else
			{
				ultimo->next=nuevo;
				ultimo=nuevo;
			}           
		}
		  
		gtk_combo_box_set_active(cbElijeDisp,0);
		free(ii);
	}
	close(socket);
	gtk_widget_destroy(dlgBuscando);     
}


void
on_enviar_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	int i;
	char MAC_dev[20]; //Direccion MAC del dispositivo detectado
	DISPOS *actual=NULL;
	GtkWidget *dlgError;
	GtkComboBox *combo;
	gchar *nombre_dispo;
	char mensaje[50];
	
	dlgEnvia=create_dlgEnvia();
	gtk_widget_show_now(dlgEnvia);
    /* Le dejo hacer sus eventos antes de buscar dispositivos */
	while (gtk_events_pending ())
    	gtk_main_iteration ();
	
	combo=GTK_COMBO_BOX(button);
	nombre_dispo=gtk_combo_box_get_active_text(combo);
	g_print("Nombre Dispositivo a enviar: %s\n",nombre_dispo);
	
	/*Ahora buscamos el nombre en la lista || search the name in the list*/
	actual=inicio;
	while (actual!=NULL)
	{
		if (strcmp(actual->nombre_dev,nombre_dispo)==0)	
		{
			strcpy(MAC_dev,actual->MAC_dev);
			actual=NULL;
		}
		else
			actual=actual->next;
	}
    	
	g_print("Direccion MAC: %s\n",MAC_dev);
	cancel=0;
	i = enviar(MAC_dev, fichero);
	
	if (dlgEnvia!=NULL)	/* Ya ha sido cerrado por otro evento || It was closed for another event */
		gtk_widget_destroy(dlgEnvia);
	
	if (i<0)
	{
		switch (i)
		{
			case -1:
				strcpy(mensaje,"Error al conectar!");
				g_print("-- Error al conectar! || sorry, unable to connect!\n");
				break;
		
			case -2:
				strcpy(mensaje,"No se puede enviar!");
				g_print("-- No se puede enviar || sorry, unable to send!\n");
				break;
			
			default:
				strcpy(mensaje,"Hubo un error al enviar");
				g_print("-- Hubo un error al enviar || sorry, An error ocurrer on send!\n");
				break;
		}
		dlgError = gtk_message_dialog_new (NULL,
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  "Error: \n%s",mensaje);
 		gtk_dialog_run (GTK_DIALOG (dlgError));
 		gtk_widget_destroy (dlgError);
                   
	}
}

void
on_txtFichero_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkEntry *txtfichero=GTK_ENTRY(editable);
	
	fichero=gtk_entry_get_text(txtfichero);
}


void
on_salir_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_acerca_de_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *ventana=create_frmAcerca ();
	gtk_widget_show (ventana);
}


void
on_cbElijeDisp_changed                 (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	GtkWidget *botonenviar=GTK_BUTTON(combobox);
	
	if(inicio==NULL)
    	gtk_widget_set_sensitive (botonenviar, FALSE);	/* DesActivo el boton */
	else
		gtk_widget_set_sensitive (botonenviar, TRUE);	/* Activo el boton */
}


void
on_frmPrincipal_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
	/* Vacio la memoria || delette the memory used by the lists */
	elimina_lista();
	gtk_main_quit();
}

void
on_dlgEspera_response                  (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{	/* Se le ha dado  al boton cerrar o se ha cerrado la ventana directamente ||
	It has been pressed the button "Close" or the window has been closed directly */	
	if (response_id==GTK_RESPONSE_CLOSE)
		gtk_widget_destroy(dlgespera);
		
	dlgespera=NULL;	
	cancel=1;
}

void
on_dlgEnvia_response                   (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
	/* Se le ha dado  al boton cerrar o se ha cerrado la ventana directamente ||
	It has been pressed the button "Close" or the window has been closed directly */	
	if (response_id==GTK_RESPONSE_CLOSE)
		gtk_widget_destroy(dlgEnvia);
		
	dlgEnvia=NULL;	
	cancel=1;
}

void
on_dlgBuscando_response                (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
	cancel=1;
}

void elimina_lista ()
{
	DISPOS *actual=inicio;
	
	while (actual!=NULL)
	{
		actual=actual->next;
		free (inicio);
		inicio=actual;
	}
}

void reset_dispos (GtkComboBox *cbElijeDisp)
{
	DISPOS *ultimo=NULL;
	int a;
	
	if (inicio!=NULL)	/* Primero elimino los elementos del combo_box y despues elimino la lista */
	{							/* First delette the elements in combo_box and later delette the list */
		ultimo=inicio;
		a=0;
		while (ultimo!=NULL)	/* La lista la recorro aqui para saber cuantos elementos existen */
		{
			gtk_combo_box_remove_text (GTK_COMBO_BOX(cbElijeDisp), 0);
			ultimo=ultimo->next;
			a++;
		}
		
		elimina_lista ();
		ultimo=NULL;
	}
}

void
on_frmAcerca_response                  (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
	/* Cierro el dialogo || close dialog*/
	GtkWidget *frmAcerca = GTK_DIALOG(dialog);
	gtk_widget_destroy(frmAcerca);
}
